/*
 Daybo Logic Standard Inclusions
 Copyright (C) 1998-2006 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
*/

#ifndef INC_COORDINS_H
#define INC_COORDINS_H
/*-------------------------------------------------------------------------*/
/*
  Standard coordinates as defined by Daybo Logic.

  This structure is for C programmers to pass simple flat coordinates
*/

/*
  If not already registered as defined by somebody else
*/
#ifndef COORD_XY_DEFINED
#define COORD_XY_DEFINED

typedef struct S_COORD_XY
{
  int X, Y;
} COORD_XY, *PCOORD_XY;

#endif /*!COORD_XY_DEFINED*/

/*-------------------------------------------------------------------------*/
/*
  This structure is for C programmer to pass X,Y,Z coodinates, a point
  which has three dimensions
*/

#ifndef COORD_XYZ_DEFINED
#define COORD_XYZ_DEFINED

typedef struct S_COORD_XYZ
{
  int X, Y, Z;
} COORD_XYZ, *PCOORD_XYZ;

#endif /*!COORDXYZ_DEFINED*/

/*-------------------------------------------------------------------------*/
#ifdef __cplusplus
/*
  This section is for C++ only.  It is a classed way of handling the
  coordinates.  If you use this section you will need to link with the
  object code as well as include the header.  The first here is a useless
  class, use it to derive cleverer classes only.
*/

#ifndef __NO_NAMESPACES__ /* Allowed to use namespaces? */
/*
  Palmer's private namespace to separate this co-ordinate from the rest
  of the team (unless they explicitly reference it)
*/
namespace DDRPalmer
{
#endif
  class TBaseCoord_XY /* Base class for XY coordinates */
  {
  protected: /* not accessable from the outside, OK for base use */
      int x, y; /* private members */
  public:
    TBaseCoord_XY(); /* Constructor (inits elements to zero) */
  };

  class TBaseCoord_XYZ /* Base class for XYZ coordinates */
  {
  protected:
      int x, y, z;
  public:
    TBaseCoord_XYZ(); /* Constructor (inits elements to zero) */
  };
/*-------------------------------------------------------------------------*/
  /* One normally uses this class to contain X,Y cordinates */
  class TCoord_XY : public TBaseCoord_XY
  {
  public: /* everything is acessable */
    // default (no params) constructor has been iherited
    TCoord_XY(TCoord_XY&); /* Copy constructor */
    TCoord_XY(COORD_XY&); /* Incase of copying from C structure */

    TCoord_XY& operator =(TCoord_XY&); /* Assignment from another */
    operator ==(TCoord_XY&); /* Comparison with another */

    void SetX(int x); /* sets X */
    void SetY(int y); /* sets Y */
    void Set(int x, int y); /* sets both with ints */

    int GetX(); /* gets X */
    int GetY(); /* gets Y */
    void Get(COORD_XY &target); /* Fills C structure with the data */
    /*
      Sends you the members, with pointers to your ints to fill
    */
    void Get(int *yourX, int *yourY); 
  };

  class TCoord_XYZ : public TBaseCoord_XYZ
  {
  public: /* everything is accessable */
    // default (no params) constructor has been iherited
    TCoord_XYZ(TCoord_XYZ&); /* Copy Constructor */
    /*
      Constructor to copy from a flat coordinate class,
      the Z will remain zero
    */
    TCoord_XYZ(TCoord_XY&);
    TCoord_XYZ(COORD_XYZ&); /* Incase of copying from C structure */
    TCoord_XYZ(COORD_XY&); /* Incase of copying from C structure (Z==0) */

    TCoord_XYZ& operator =(TCoord_XYZ&); /* Assignment from another */
    operator ==(TCoord_XYZ&); /* Comparison with another */

    void SetX(int x); /* sets X */
    void SetY(int y); /* sets Y */
    void SetZ(int z); /* sets Z */
    void Set(int x, int y, int z); /* Sets all with ints */

    int GetX(); /* gets X */
    int GetY(); /* gets Y */
    int GetZ(); /* gets Z */
    void Get(COORD_XYZ &target); /* Fills C structure with the data */
    /*
      Sends you the members, with pointers to your ints to fill
    */
    void Get(int *yourX, int *yourY, int *yourZ);
  };
#ifndef __NO_NAMESPACES__ /* Allowed to use namespaces? */
} /* end namespaces DDRPalmer */
#endif

#endif /*__cplusplus : end of C++ only section */
/*-------------------------------------------------------------------------*/
#endif /*!INC_COORDINS_H*/