/*
 Daybo Logic Standard Inclusions
 Copyright (C) 1998-2006 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.

 The Daybo Logic standards inclusion file for C/C++, all Daybo Logic
 standards must be defined here (or another file included at the extra
 inclusions section which is defined right at the end of the file.  Any
 changes that are to be made here cannot be automatically done.  Please
 speak to David Duncan Ross Palmer if you wish to make amendments to the
 header, as any changes will mean a rebuild of the entire system.  This
 is so because (nearly) every Daybo Logic package or product  includes
 this header.  Anyone who doesn't include this header may break
 standards and will not be able to tell where they are at compile
 time (because of conditional defines).

 If you have created a header you wish to add to the dlstdinc.h by an
 inclusion in the privileged inclusion section you must obtain
 permission from David Duncan Ross Palmer.  Please remember that once it
 has obtained this privilege it inherits the same status as the header
 and the header is dependent upon it, therefore it must not be modified
 without permission either as this would result in a rebuild of the
 entire system.

 Original creation of header file - David Duncan Ross Palmer

Notes on inclusion:
 (A) Warning!  If you are going to include windows.h you must do so
  before the inclusion of this header so that I know where I am and
  what to define (such as types that might have been declared by
  Windows).

Increment this revision number on any change of the header but not
of it's dependencies.

$Id: dlstdinc.h,v 38.4 2006/02/20 09:45:00 ddrp Exp $


Just in case identifiers conflict, to prevent unneccesary changes to this
file, these identifiers can be defined before the inclusion of this headers
to disable elements.  First it the identifier, then the effects:

  DLSTDINC_NOBCBSPECIFICS - Stops C++Builder inclusions specifically even if
                            compiling under C++Builder.

  DLSTDINC_NOCBOOLEAN - Turns off C boolean types specifically, then BOOL,
                        FALSE, TRUE etc. aren't defined here.

  DLSTDINC_NOMETRICS - No metrics, anything to do with metrics and dimensions
                       won't be mentioned, we won't even dare to say it!

  DLSTDINC_NOMINMAX - Stops redefinition of min max macros.

  DLSTDINC_NOPTRABSTRACTIONS - Stops defintion of things like VOID or FAR in
                               case they somehow conflict with Windows or
                               something.

  DLSTDINC_NOENDIAN - No endian macros or code
*/

#ifndef INC_DLSTDINC_H
#define INC_DLSTDINC_H /* Multiple inclusion sentry */
/*-------------------------------------------------------------------------*/
/*  Main environment realisation begins here */

/* First we must know the compiler of our world */

#if defined(__BCB__) /* Borland C++Builder? */

#elif defined(__BCW_5__) /* No, Borland C++ 5? */

#elif defined(__BCW_45__) /* No, Borland C++ 4.5?*/

#elif defined (__TC__) /* No, Turbo C */

#elif defined(__MVC__) /* No, Visual C++? */

#elif defined(__GNUC__) /* No, GNU C? */

#elif defined(__WATCOMC__) /* No, Watcom C? */

#else
#  error "Must define compiler or add compiler information to standards"
#endif

#define __DAYBO_DEBUGGING__ please_replace_this_macro_with_DEBUG
#ifndef NDEBUG /* No debugging has NOT been defined? */
# define DEBUG /* Debug mode */
#endif /*NDEBUG*/
/*-------------------------------------------------------------------------*/
/*
  Priority declarations section.
  Some of the header files that are included may rely on types that are
  standards.  Most types are declared in extra inclusions at the bottom.
  However if you need to declare something at the top of this header,
  here is the place to do it.  This is usaully used because following
  headers need it
*/

#if \
  ( \
    !defined(DLSTDINC_NOENDIAN) && \
    !defined(_WINDEF_) && \
    !defined(_WINDEF_H) \
  )

# if !defined(__B_ENDIAN__) && !defined (__L_ENDIAN__)
/* Use default, emit warning */
#   pragma msg ("Assuming __L_ENDIAN__")
#   define __L_ENDIAN__
# endif

# ifdef __B_ENDIAN__ /* Big endian CPU (such as Mac) - MOTOROLA */
#   define MAKEWORD(a, b)  \
      (((WORD)((BYTE)(b))) << 8) | ((WORD)((BYTE)(a)))
#   define MAKEDW(a, b) \
      (((DWORD)((WORD)(b))) << 16) | ((DWORD)((WORD)(a)))

#   define LOWORD(l)          ((WORD)(((DWORD)(l) >> 16) & 0xFFFF))
#   define HIWORD(l)          ((WORD)(l))
#   define LOBYTE(w)          ((BYTE)(((WORD)(w) >> 8) & 0xFF))
#   define HIBYTE(w)          ((BYTE)(w))
# else /* Little endian CPU (such as I.B.M. P.C.) - INTEL */
#   define MAKEWORD(a, b) \
      ((WORD)(((BYTE)(a)) | (((WORD)((BYTE)(b))) << 8)))
#   define MAKEDW(a, b) \
      ((DWORD)(((WORD)(a)) | (((DWORD)((WORD)(b))) << 16)))
#   define LOWORD(l)         ((WORD)(l))
#   define HIWORD(l)         ((WORD)(((DWORD)(l) >> 16) & 0xFFFF))
#   define LOBYTE(w)         ((BYTE)(w))
#   define HIBYTE(w)         ((BYTE)(((WORD)(w) >> 8) & 0xFF))
# endif

#endif /*!_WINDEF_ && !DLSTDINC_NOENDIAN */

/* Priority inclusions section.
Same as priority declarations except the declarations are kept in extra header
files.  NOTE:  Headers included here must not rely heavily on the standards
heavily as very few have been defined as these headers have priority, they
have been compiled first. */

/*
  Include headers _here_
*/
/*-------------------------------------------------------------------------*/
/* Conditional inclusions for Borland C++Builder */
#if defined(__BCB__) && !defined(DLSTDINC_NOBCBSPECIFICS)
/*#  include "" */
#endif /*__BCB__*/
/*-------------------------------------------------------------------------*/
/* Conditional inclusions for Borland C++ 4.5 specifically */
#ifdef __BCW_45__
/*#  include "" */
#endif /*__BCW_45__*/
/*-------------------------------------------------------------------------*/
/* Conditional inclusions for Borland C++ 5 specifically */
#ifdef __BCW_5__
/*#  include "" */
#endif /*__BCW_5__*/
/*-------------------------------------------------------------------------*/
/* Conditional inclusions for Borland C++ (NOT BUILDER) in general */
#if defined(__BCW_5__) || defined(__BCW_45__)
/*#  include "" */
#endif /*__BCW_5__ || __BCW_45__*/
/*-------------------------------------------------------------------------*/
/* Privelleged extra inclusions section, this to be modified with permission
only.  Do not change the files listed either (or their modification dates).

Each header must be followed by it's author(s) names */

#ifndef DLSTDINC_NOCBOOLEAN
  /* New method for defining stdbool. */
#  include "dlstdinc_cboolean.h" /* David Duncan Ross Palmer */
#endif
#include "dlstdinc_pcintern.h" /* M. Tischer/B. Jennrich/D. Palmer */
#include "dlstdinc_morelmts.h" /* David Duncan Ross Palmer */
#include "dlstdinc_inttypes.h" /* inttypes.h/stdint.h etc */
#include "dlstdinc_dlrnd.h" /* David Duncan Ross Palmer */
#ifndef DLSTDINC_NOMINMAX
#  include "dlstdinc_minmax.h" /* David Duncan Ross Palmer */
#endif
#ifndef DLSTDINC_NOPTRABSTRACTIONS
#  include "dlstdinc_farptr.h" /* David Duncan Ross Palmer */
#  include "dlstdinc_voidptr.h" /* David Duncan Ross Palmer */
#endif
/*-------------------------------------------------------------------------*/
/*
  Final types.  These privileged types are not relied apon by the
  inclusions but are used by Daybo Logic DLLs and the like
*/

#define GUIDLENGTH (38) /* A GUID in a full string format */

/*
  S_DL_FUNCVER - structure for reading a function version
*/
typedef struct _S_DL_FUNCVER
{
  WORD size; /* sizeof() this strucure */
  BYTE major; /* Major version number of function */
  BYTE minor; /* Minor version number of function */
  BOOL debug; /* Indicates TRUE if function not bug-free yet */
} S_DL_FUNCVER, *S_DL_PFUNCVER;

/*
  S_DL_MODVER - Structure for reading a module version
*/
typedef struct _S_DL_MODVER
{
  WORD size; /* sizeof() this strucutre */
  BYTE major; /* Major version number of module */
  BYTE minor; /* Minor version number */
  WORD build; /* Build # of version of module */
} S_DL_MODVER, *S_DL_PMODVER;

#define DL_MODVEREX_OS_PORTABLE                 (-1)
#define DL_MODVEREX_OS_MSDOS                     (0)
#define DL_MODVEREX_OS_WIN16                     (1)
#define DL_MODVEREX_OS_WIN32                     (2)
#define DL_MODVEREX_OS_WINNT_ONLY                (3)
#define DL_MODVEREX_OS_MAC                       (4)
#define DL_MODVEREX_OS_OS2                       (5)
#define DL_MODVEREX_OS_OS2WARP                   (6)
#define DL_MODVEREX_OS_UNKNOWN                   (7)
#define DL_MODVEREX_OS_UNIX                      (8)
#define DL_MODVEREX_OS_MAC_OSX                   (9)
#define DL_MODVEREX_OS_LINUX                    (10)
#define DL_MODVEREX_OS_BSD                      (11)
#define DL_MODVEREX_OS_FREEBSD                  (12)
#define DL_MODVEREX_OS_NETBSD                   (13)
#define DL_MODVEREX_OS_OPENBSD                  (14)
#define DL_MODVEREX_OS_IBSD                     (15)
#define DL_MODVEREX_OS_CPM                      (16)
#define DL_MODVEREX_OS_SOLARIS                  (17)
#define DL_MODVEREX_OS_SCO                      (18)
#define DL_MODVEREX_OS_SCO_OPENSERVER           (19)

/*
  D_DL_MODVER_EX - Extended structure for reading a module version
*/
typedef struct _S_DL_MODVER_EX
{
  WORD size; /* sizeof() this structure */
  BYTE major; /* Major version number of modile */
  BYTE minor; /* Minor version number of module */
  WORD build; /* Build # of version of module */
  BOOL debug; /* if TRUE this module is a debugging version */
  char author[128]; /* Author(s) of module */
  char desc[2048]; /* Short description of module */
  char product[512]; /* Product (if file is part of a larger package) */
  /*
    If the source is OS specific this specifies it, or -1 (Portable)
  */
  char sourcebasedOS;
  /*
    Each module ignoring the portability of the source is built for an
    OS, this specifies it and should not be set to -1 (Portable)
  */
  char requiredOS;
} S_DL_MODVER_EX, *S_DL_PMODVER_EX;

/*
  As long as metrics are even allowed to be mentioned
*/
#ifndef DLSTDINC_NOMETRICS
/*
  Structure for quickly passing all positioning attributes of a window
*/
typedef struct _S_DL_MSWINDOWS_WINDOW_METRICS
{
  WORD size; /* sizeof() this structure */
  int left;
  int top;
  int width;
  int height;
} S_DL_MSWINDOWS_WINDOW_METRICS, *S_DL_MSWINDOWS_PWINDOW_METRICS;
#endif /*DLSTDINC_NOMETRICS*/
/*-------------------------------------------------------------------------*/
#endif /*!INC_DLSTDINC_H*/

/*
  End of Protected Standard inclusions header file - DO NOT MODIFY WITHOUT
  PERMISSION, read notice at top of file.
*/
