/*
 Daybo Logic Standard Inclusions
 Copyright (C) 1999-2006 Daybo Logic.
 All rights reserved.

 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in the
    documentation and/or other materials provided with the distribution.
 3. Neither the name of the project nor the names of its contributors
    may be used to endorse or promote products derived from this software
    without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
 FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 SUCH DAMAGE.
*/

#ifndef INC_DLSTDINC_FARPTR_H
#define INC_DLSTDINC_FARPTR_H
/*-------------------------------------------------------------------------*/
/*
Use the macro FAR to shortcut the fact that the type FAR is non-ANSI,
the type FAR is converted to far if in non-ANSI mode otherwise the
FAR is deleted before compilation by definition to nothing.  Never use
far or FAR in code that could be ported to another platform, use FAR
to ensure if you are writing for the PC (Intel) in 16-bit segmented
mode that if you use an ANSI C compiler the code will still compile.

If you are relying on the fact that a pointer is far rather than suggesting
it to the compiler, for example BIOS variable changing code, your code
may fail when compiled on an ANSI compiler unless all pointers are made
far by selecting an appropriate large memory model */

#ifndef FAR /* Do this as long as nobody else defined FAR */
# ifdef __STDC__ /* Standard ANSI C compiler? */
#   define FAR /* FAR compiles but to nothing */
# else /* non-standard compiler */
    /*
      Use 'far' keyword and hope compiler likes it, if code error happens
      here don't use far pointers with this compiler
    */
#   define FAR far
# endif /*__STDC__*/
#endif /*!FAR*/
/*-------------------------------------------------------------------------*/
#endif /*!INC_DLSTDINC_FARPTR_H*/
