/*
$Id: dlstdinc_stdint.h,v 0.3 2006/02/20 09:46:00 ddrp Exp $

Copyright (C) 2001-2006 Daybo Logic.
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.
3. Neither the name of the project nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE PROJECT AND CONTRIBUTORS ``AS IS'' AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED.  IN NO EVENT SHALL THE PROJECT OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.
*/
/*
  These are a few of my replacements incase you have no stdint.h.  It is
  not a complete replacement as I don't know the environment like the compiler
  does, I don't know if your system can support 64-bit integers and such
  like the _read_ stdint.h does, only use this if there is no proper C99
  standard stdint.h.  This file should not be used if stdint.h or inttypes.h
  have already been included because we have no standard way to tell if they
  have so we will fail with redefinitions.  The macros that go with these
  types should not ever be used anywhere in OverChat because, although the
  standard (C99) says inttypes.h will include stdint.h, on FreeBSD etc.,
  where there is no stdint.h at time of writing, inttypes.h won't make
  them available.  Conclusion: If HAVE_STDINT_H, include stdint.h, if
  HAVE_INTTYPES_H, include inttypes.h, if not fall back on this header.
  Note: STDINT_NO_BASE_TYPES was invented by me, "base types" are defined
  as anything which might be defined by inttypes.h.  This is quite useful
  if you need things like int_least16_t but not the uint16_t types.
    -- DDRP <daybologic.co.uk/mailddrp>
*/

#ifndef _STDINT_H
#define _STDINT_H       1
/*-------------------------------------------------------------------------*/
#include <limits.h>
/*
  Some error checks before we begin
*/
/*
  Some error checks before we begin the next definition
*/
# ifndef UINT_MAX
#  error ("Must have ANSI compliant limits.h with UINT_MAX if not HAVE_STDINT_H")
# endif /*!UINT_MAX*/

# ifndef ULONG_MAX
#  error ("Must have ANSI compliant limits.h with ULONG_MAX if not HAVE_STDINT_")
# endif /*!ULONG_MAX*/

# ifndef UCHAR_MAX
#  error ("Must have ANSI compliant limits.h with UCHAR_MAX if not HAVE_STDINT_H")
# endif /*!UCHAR_MAX*/

# ifndef SHRT_MAX
#  error ("Must have ANSI compliant limits.h with SHRT_MAX if not HAVE_STDINT_H")
# endif /*!SHRT_MAX*/

# ifndef INT_MAX
#  error ("Must have ANSI compliant limits.h with INT_MAX if not HAVE_STDINT_H")
# endif /*!INT_MAX*/

# ifndef LONG_MAX
#  error ("Must have ANSI compliant limits.h with LONG_MAX is not HAVE_STDINT_H")
# endif /*!LONG_MAX*/

/*
  Declaration of my backup uint8_t (backup 8 bit integer)
*/
# ifndef STDINT_NO_BASE_TYPES
#  if ( UCHAR_MAX == 255 )
    typedef unsigned char uint8_t;
#  else
#   error ("unsigned char is not 8bits, define HAVE_STDINT_H?")
#  endif
# endif /*!STDINT_NO_BASE_TYPES*/

/*
  Declaration of my backup version of uint32_t, 32-bit integer
*/
# ifndef STDINT_NO_BASE_TYPES
#  if ( ULONG_MAX == 0xFFFFFFFFUL )
    typedef unsigned long uint32_t;
#  elif ( UINT_MAX == 0xFFFFFFFFU ) /* 64-bit word it seems */
    typedef unsigned int uint32_t;
#  else
#   error ("Cannot determine a definitive uint32_t")
#  endif
# endif /*!STDINT_NO_BASE_TYPES*/

/*
  Declaration of int_least16_t (at least 16 bits)
*/
# if ( SHRT_MAX >= 0x7FFF )
   typedef signed short int int_least16_t;
# elif ( INT_MAX >= 0x7FFF )
   typedef signed int int_least16_t;
# elif ( LONG_MAX >= 0x7FFF )
   typedef signed long int int_least16_t;
# else
#  error ("No known type is at least 16-bits neccersary for int_least16_t")
# endif
/*-------------------------------------------------------------------------*/
#endif /*!_STDINT_H*/
